/*  Copyright 2001 Terje Sannum <terje@waste.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
*/

#include "sms.h"
#include <PalmOS.h>
#include <SerialMgr.h>

/* App preferences */
typedef struct {
  Int8 port, speed, bits, parity, stopbits, flow;
  char modemInit[101];
  char dialPrefix[11];
  char sender[21];
  char reciever[21];
  char message[161];
  char smsc[21];
} AppPrefType;
  
FormPtr smsForm, serialForm; 
AppPrefType prefs;

static setResponse(char *text) {
  FieldPtr fld;
  MemHandle m;
  MemPtr mp;
  fld = FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSResponse));
  FldFreeMemory(fld);
  m = MemHandleNew(StrLen(text)+1);
  mp = MemHandleLock(m);
  StrCopy(mp, text);
  MemPtrUnlock(mp);
  FldSetText(fld, m, 0, MemHandleSize(m));
  FldDrawField(fld);
}

static Err serialSend(UInt16 portId, char *text) {
  Err err;
  int numSent;
  numSent = SrmSend(portId, text, StrLen(text), &err); 
  return err;
} 

static Err serialRead(UInt16 thePort, UInt8 *rcvDataP, char eof, int secs) { 
  UInt32 bytesRcvd, waitTime, totalRcvBytes = 0; 
  UInt8 *newRcvBuffer; 
  Err err; 
  waitTime = SysTicksPerSecond() * secs; 
  do { 
    bytesRcvd = SrmReceive(thePort, rcvDataP, 1, waitTime, &err); 
    totalRcvBytes += bytesRcvd; 
    rcvDataP += bytesRcvd;
    if ((err == serErrTimeOut) && (bytesRcvd == 0)) 
      break;
    if ((err) && (err != serErrTimeOut)) 
      break;
  } while (*(rcvDataP-1) != eof); 
  *rcvDataP = 0;
  return err; 
} 

static int checksum(char *str) {
  char	*ptr;
  int	j = 0;
  for (ptr = str; *ptr != '\0'; ptr++) {
    j += *ptr;
    if (j >= 256) j -= 256;
  }
  return j;
}

/* charset conversion table - unsupported characters are changed to
   space (0x20) */
static char charset[] = { 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x0a /*   */, 0x20 /*   */,
  0x20 /*   */, 0x0d /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x21 /* ! */, 0x22 /* " */, 0x23 /* # */,
  0x03 /* $ */, 0x25 /* % */, 0x26 /* & */, 0x27 /* ' */,
  0x28 /* ( */, 0x29 /* ) */, 0x2a /* * */, 0x2b /* + */,
  0x2c /* , */, 0x2d /* - */, 0x2e /* . */, 0x2f /* / */,
  0x30 /* 0 */, 0x31 /* 1 */, 0x32 /* 2 */, 0x33 /* 3 */,
  0x34 /* 4 */, 0x35 /* 5 */, 0x36 /* 6 */, 0x37 /* 7 */,
  0x38 /* 8 */, 0x39 /* 9 */, 0x3a /* : */, 0x3b /* ; */,
  0x3c /* < */, 0x3d /* = */, 0x3e /* > */, 0x3f /* ? */,
  0x00 /* @ */, 0x41 /* A */, 0x42 /* B */, 0x43 /* C */,
  0x44 /* D */, 0x45 /* E */, 0x46 /* F */, 0x47 /* G */,
  0x48 /* H */, 0x49 /* I */, 0x4a /* J */, 0x4b /* K */,
  0x4c /* L */, 0x4d /* M */, 0x4e /* N */, 0x4f /* O */,
  0x50 /* P */, 0x51 /* Q */, 0x52 /* R */, 0x53 /* S */,
  0x54 /* T */, 0x55 /* U */, 0x56 /* V */, 0x57 /* W */,
  0x58 /* X */, 0x59 /* Y */, 0x5a /* Z */, 0x20 /* [ */,
  0x20 /* \ */, 0x20 /* ] */, 0x14 /* ^ */, 0x20 /* _ */,
  0x20 /*   */, 0x61 /* a */, 0x62 /* b */, 0x63 /* c */,
  0x64 /* d */, 0x65 /* e */, 0x66 /* f */, 0x67 /* g */,
  0x68 /* h */, 0x69 /* i */, 0x6a /* j */, 0x6b /* k */,
  0x6c /* l */, 0x6d /* m */, 0x6e /* n */, 0x6f /* o */,
  0x70 /* p */, 0x71 /* q */, 0x72 /* r */, 0x73 /* s */,
  0x74 /* t */, 0x75 /* u */, 0x76 /* v */, 0x77 /* w */,
  0x78 /* x */, 0x79 /* y */, 0x7a /* z */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */,
  0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 0x20 /*   */, 
  0x20 /*   */, 0x40 /*  */, 0x20 /*  */, 0x01 /*  */, 
  0x24 /*  */, 0x03 /*  */, 0x20 /*  */, 0x5f /*  */,
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x40 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x40 /*  */, 0x20 /*  */, 0x60 /*  */,
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x5b /*  */, 0x0e /*  */, 0x1c /*  */, 0x09 /*  */,
  0x20 /*  */, 0x1f /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x5d /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x20 /*  */, 0x5c /*  */, 0x20 /*  */,
  0x0b /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x5e /*  */, 0x20 /*  */, 0x20 /*  */, 0x1e /*  */,
  0x7f /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x7b /*  */, 0x0f /*  */, 0x1d /*  */, 0x20 /*  */,
  0x04 /*  */, 0x05 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x07 /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x7d /*  */, 0x08 /*  */, 0x20 /*  */,
  0x20 /*  */, 0x20 /*  */, 0x7c /*  */, 0x20 /*  */,
  0x0c /*  */, 0x06 /*  */, 0x20 /*  */, 0x20 /*  */,
  0x7e /*  */, 0x20 /*  */, 0x20 /*  */, 0x20 /*  */
};


static void hexMessage(char *dest) {
  int i, j = 0;
  char *p, hex[5];
  for(p = prefs.message; *p != '\0'; p++) {
    StrPrintF(hex, "%x", (int)charset[(unsigned char)*p]);
    StrCopy(dest+j*2, hex+2);
    j++;
  }
}

static int checkResponse(char *response) {
  if(*response == 0x02 && *(response+15) == 'A') return 1;
  else return 0;
}

static void generateMessage(char *dest) {
  char hex[5];
  int i;
  MemHandle hexMH,msgMH;
  MemPtr hexMP,msgMP;
  hexMH = MemHandleNew(StrLen(prefs.message)*2);
  hexMP = MemHandleLock(hexMH);
  msgMH = MemHandleNew(1024);
  msgMP = MemHandleLock(msgMH);
  hexMessage(hexMP);
  StrPrintF(msgMP, "%02d/%05d/%s/01/%s/%s//3/%s/%c", 1, 22+StrLen(prefs.reciever)+StrLen(prefs.sender)+StrLen(hexMP), "O", prefs.reciever, prefs.sender, hexMP, 0x00);
  StrPrintF(hex, "%x", checksum(msgMP));
  StrPrintF(dest, "%c%s%s%c%c", 0x02, msgMP, hex+2, 0x03, 0x00);
  MemHandleUnlock(hexMH);
  MemHandleUnlock(msgMH);
}

static void serialSendMessage() {
  Err err;
  UInt16 portId, settingsSize;
  UInt32 value;
  Int32 speed, settings;
  char rcv[512];
  char snd[1024];
  /* check for new serial manager */
  err = FtrGet(sysFileCSerialMgr, sysFtrNewSerialPresent, &value);
  if(err || !value) {
    FrmCustomAlert(SerialAlert, "New serial manager not installed.", NULL, NULL);
    return;
  }
  if(prefs.speed == 0) speed = 300;
  else if(prefs.speed == 1) speed = 1200;
  else if(prefs.speed == 2) speed = 2400;
  else if(prefs.speed == 3) speed = 9600;
  else if(prefs.speed == 4) speed = 19200;
  else if(prefs.speed == 5) speed = 28800;
  else if(prefs.speed == 6) speed = 38400;
  else if(prefs.speed == 7) speed = 57600;
  else if(prefs.speed == 8) speed = 111200;
  else speed = 9600;
  if(prefs.port == SerPortIR) {
    err = SrmOpen(sysFileCVirtIrComm, speed, &portId);
  } else {
    err = SrmOpen(serPortCradlePort, speed, &portId);
  }
  if(err != 0) {
    FrmCustomAlert(SerialAlert, "Can't open serial port.", NULL, NULL);
    return;
  }
  /* change settings according to preferences */
  settings = srmSettingsFlagRTSAutoM | srmSettingsFlagCTSAutoM; 
  if(prefs.bits == SerPort8Bit) settings |= srmSettingsFlagBitsPerChar8;
  else settings |= srmSettingsFlagBitsPerChar7;
  if(prefs.parity == SerPortPOdd) settings |= srmSettingsFlagParityOnM;
  else if(prefs.parity == SerPortPEven) 
    settings |= (srmSettingsFlagParityOnM | srmSettingsFlagParityEvenM);
  if(prefs.stopbits == SerPortStop1) 
    settings |= (srmSettingsFlagStopBitsM | srmSettingsFlagStopBits1);
  else if(prefs.stopbits == SerPortStop2) 
    settings |= (srmSettingsFlagStopBitsM | srmSettingsFlagStopBits2);
  settingsSize = sizeof(settings);
  err = SrmControl(portId, srmCtlSetFlags, &settings, &settingsSize);
  if(err != errNone) {
    FrmCustomAlert(SerialAlert, "Serial port configuration failed.", NULL, NULL);
    SrmClose(portId);
    return;
  }
  /* start sending */
  SrmReceiveFlush(portId, 0);
  setResponse("checking modem...");
  err = serialSend(portId, "AT\r\n");
  err = serialRead(portId, rcv, '\n', 3);
  if(!StrNCompare(rcv, "AT", 2)) err = serialRead(portId, rcv, '\n', 3);
  if(*rcv == '\r' || *rcv == '\n') err = serialRead(portId, rcv, '\n', 2); 
  if(StrNCompare(rcv, "OK", 2)) {
    setResponse("Illegal or no modem response.");
    SrmClose(portId);
    return;
  }
  setResponse("dialing...");
  err = serialSend(portId, "ATD");
  err = serialSend(portId, prefs.smsc);
  err = serialSend(portId, "\r\n");
  err = serialRead(portId, rcv, '\n', 20);  
  if(!StrNCompare(rcv, "ATD", 3)) err = serialRead(portId, rcv, '\n', 20); 
  if(*rcv == '\r' || *rcv == '\n') err = serialRead(portId, rcv, '\n', 2); 
  if(!StrNCompare(rcv, "CARRIER", 7)) {
    setResponse("got carrier...");
    err = serialRead(portId, rcv, '\n', 20); 
    if(*rcv == '\r' || *rcv == '\n') err = serialRead(portId, rcv, '\n', 2); 
  }
  if(StrNCompare(rcv, "CONNECT", 7)) {
    setResponse("Connect error.");
    SrmClose(portId);
    return;
  }
  setResponse("connected...");
  generateMessage(snd);
  err = serialSend(portId, snd); 
  err = serialRead(portId, rcv, 0x03, 3); 
  if(checkResponse(rcv)) setResponse("Message acknowleged.");
  else setResponse("Bad SMSC response.");
  err = serialSend(portId, "+++ATH0\r\n");
  SrmClose(portId);
}

static void sendMessage() {
  if(prefs.sender == NULL || StrLen(prefs.sender) == 0) {
    FrmCustomAlert(InputAlert, "Sender not specified.", NULL, NULL);
    return;
  }
  if(prefs.reciever == NULL || StrLen(prefs.reciever) == 0) {
    FrmCustomAlert(InputAlert, "Reciever not specified.", NULL, NULL);
    return;
  }
  if(prefs.message == NULL || StrLen(prefs.message) == 0) {
    FrmCustomAlert(InputAlert, "No message.", NULL, NULL);
    return;
  }
  if(prefs.smsc == NULL || StrLen(prefs.smsc) == 0) {
    FrmCustomAlert(InputAlert, "SMSC not specified.", NULL, NULL);
    return;
  }
  /* send message on serial */
  serialSendMessage();
}

static void SMSFormGetValues() {
  char *p;
  /* sender */
  p = FldGetTextPtr(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSSender)));
  if(p != NULL && StrLen(p)>0) {
    StrCopy(prefs.sender, p);
    *(prefs.sender+StrLen(p)+1) = 0x00;
  } else prefs.sender[0] = '\0';
  /* reciever */
  p = FldGetTextPtr(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSReciever)));
  if(p != NULL && StrLen(p)>0) {
    StrCopy(prefs.reciever, p);
    *(prefs.reciever+StrLen(p)+1) = 0x00;
  } else prefs.reciever[0] = '\0';
  /* message */
  p = FldGetTextPtr(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSText)));
  if(p != NULL && StrLen(p)>0) {
    StrCopy(prefs.message, p);
    *(prefs.message+StrLen(p)+1) = 0x00;
  } else prefs.message[0] = '\0';
  /* smsc */
  p = FldGetTextPtr(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSC)));
  if(p != NULL && StrLen(p)>0) {
    StrCopy(prefs.smsc, p);
    *(prefs.smsc+StrLen(p)+1) = 0x00;
  } else prefs.smsc[0] = '\0';
}

static Boolean SMSFormHandleEvent(EventPtr event) { 
  int objId;
  Boolean handled=false; 
  switch(event->eType) { 
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == SMSSend) {
      SMSFormGetValues();
      sendMessage();
      handled = true;
    }
    break;
  case menuEvent:
    if (event->data.menu.itemID == Cut) {
      objId = FrmGetFocus(smsForm);
      FldCut(FrmGetObjectPtr(smsForm, objId));
    } else if (event->data.menu.itemID == Copy) {
      objId = FrmGetFocus(smsForm);
      FldCopy(FrmGetObjectPtr(smsForm, objId));
    } else if (event->data.menu.itemID == Paste) {
      objId = FrmGetFocus(smsForm);
      FldPaste(FrmGetObjectPtr(smsForm, objId));
    } else if (event->data.menu.itemID == Serial) {
      FrmGotoForm(SerialForm); 
    } else if (event->data.menu.itemID == About) {
      FrmAlert(AboutAlert);
    } else if (event->data.menu.itemID == Help) {
      FrmHelp(HelpText);
    }
    handled = true;
    break;
  case frmOpenEvent: 
    smsForm=FrmGetActiveForm(); 
    if ((prefs.sender != NULL) && (StrLen(prefs.sender)>0)) {
      FldInsert(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSSender)), prefs.sender, StrLen(prefs.sender));
    }
    if ((prefs.reciever != NULL) && (StrLen(prefs.reciever)>0)) {
      FldInsert(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSReciever)), prefs.reciever, StrLen(prefs.reciever));
    }
    if ((prefs.message != NULL) && (StrLen(prefs.message)>0)) {
      FldInsert(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSText)), prefs.message, StrLen(prefs.message));
    }
    if ((prefs.smsc != NULL) && (StrLen(prefs.smsc)>0)) {
      FldInsert(FrmGetObjectPtr(smsForm, FrmGetObjectIndex(smsForm, SMSC)), prefs.smsc, StrLen(prefs.smsc));
    }
    FrmDrawForm(smsForm); 
    handled=true; 
    break; 
  case frmCloseEvent: 
    SMSFormGetValues();
    FrmEraseForm(smsForm); 
    FrmDeleteForm(smsForm); 
    smsForm = 0; 
    handled = true; 
    break; 
  } 
  return handled; 
} 

static Boolean SerialFormHandleEvent(EventPtr event) { 
  int objId;
  MemHandle recordMH;
  MemPtr lp;
  char *smscRec;
  Boolean handled=false; 
  switch(event->eType) { 
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID == SerialOK) {
      FrmGotoForm(SMSForm); 
      handled = true;
    }
    break;
  case frmOpenEvent: 
    serialForm=FrmGetActiveForm(); 
    if(prefs.port == SerPortCradle) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialCradle)), 1);
    } else {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialIR)), 1);
    }
    lp = FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialSpeedList));
    LstSetSelection(lp, prefs.speed);
    CtlSetLabel(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialSpeedTrigger)), LstGetSelectionText(lp, prefs.speed));
    if(prefs.bits == SerPort8Bit) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, Serial8Bit)), 1);
    } else {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, Serial7Bit)), 1);
    }
    if(prefs.parity == SerPortPNone) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialPNone)), 1);
    } else if(prefs.parity == SerPortPOdd) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialPOdd)), 1);
    } else {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialPEven)), 1);
    }
    if(prefs.stopbits == SerPortStop0) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialStop0)), 1);
    } else if(prefs.stopbits == SerPortStop1) {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialStop1)), 1);
    } else {
      CtlSetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialStop2)), 1);
    }
    FrmDrawForm(serialForm); 
    handled=true; 
    break; 
  case frmCloseEvent: 
    if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialCradle)))) prefs.port = SerPortCradle;
    else prefs.port = SerPortIR;
    prefs.speed = LstGetSelection(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialSpeedList)));
    if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, Serial7Bit)))) prefs.bits = SerPort7Bit;
    else prefs.bits = SerPort8Bit;
    if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialPNone)))) prefs.parity = SerPortPNone;
    else if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialPOdd)))) prefs.parity = SerPortPOdd;
    else prefs.parity = SerPortPEven;
    if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialStop0)))) prefs.stopbits = SerPortStop0;
    else if(CtlGetValue(FrmGetObjectPtr(serialForm, FrmGetObjectIndex(serialForm, SerialStop1)))) prefs.stopbits = SerPortStop1;
    else prefs.stopbits = SerPortStop2;
    FrmEraseForm(serialForm); 
    FrmDeleteForm(serialForm); 
    serialForm = 0; 
    handled = true; 
    break; 
  } 
  return handled; 
} 

static Boolean ApplicationHandleEvent(EventPtr event) { 
  FormPtr frm; 
  UInt16 formId; 
  Boolean handled=false; 

  switch(event->eType) { 
  case frmLoadEvent: {
    UInt16 formId = event->data.frmLoad.formID;
    FormPtr pForm=FrmInitForm(formId); 
    FrmSetActiveForm(pForm); 
    if(formId == SMSForm) FrmSetEventHandler(pForm, SMSFormHandleEvent); 
    else FrmSetEventHandler(pForm, SerialFormHandleEvent);
    handled = true; 
    break; 
  }
  default: 
    break; 
 } 
 return handled;  
} 

static void EventLoop() { 
  EventType event; 
  UInt16 error; 
  do  {  
    EvtGetEvent(&event,evtWaitForever); 
    if(!SysHandleEvent(&event)) 
      if(!MenuHandleEvent(0,&event,&error)) 
	if(!ApplicationHandleEvent(&event)) 
	  FrmDispatchEvent(&event); 
  } while(event.eType!=appStopEvent); 
} 

static Err StartApplication() { 
  Err err = 0;
  UInt16 prefsSize = sizeof(AppPrefType);
  if(PrefGetAppPreferences('SMSs', 0, &prefs, &prefsSize, true) == noPreferenceFound) {
    prefs.port = SerPortIR;
    prefs.speed = 3;
    prefs.bits = SerPort8Bit;
    prefs.parity = SerPortPNone;
    prefs.stopbits = SerPortStop1;
    prefs.sender[0] = '\0';
    prefs.reciever[0] = '\0';
    prefs.message[0] = '\0';
    prefs.smsc[0] = '\0';
  }
  FrmGotoForm(SMSForm); 
  return 0; 
} 

static void StopApplication() { 
  FrmCloseAllForms(); 
  PrefSetAppPreferences('SMSs', 0, 1, &prefs, sizeof(AppPrefType), true);
} 

UInt32 PilotMain(UInt16 launchCode, void *cmdPBP, UInt16 launchFlags) { 
  Err err=0; 
  if(launchCode==sysAppLaunchCmdNormalLaunch) { 
    if((err=StartApplication())==0) { 
      EventLoop(); 
      StopApplication(); 
    } 
  } 
  return err; 
}
